/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file ffstream.cc implements ffstream.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif


// std c
#include <stdio.h>

// std c++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <string>
using std::string;

// this header
#include "ffostream.hh"

// PIAVE
#include "libpiave/frame.hh"

using namespace PIAVE;

/*
 *  class OutFFMpegStream 
 */
OutFFMpegStream::OutFFMpegStream( const string& fileName )
  : OutAVFStream( fileName )
{
    FFMpeg::init();
    initEncoder(  );
    return;
}

OutFFMpegStream::~OutFFMpegStream()
{
    /* add sequence end code to have a real mpeg file */
    _outbuf[0] = 0x00;
    _outbuf[1] = 0x00;
    _outbuf[2] = 0x01;
    _outbuf[3] = 0xb7;
    fwrite( _outbuf, 1, 4, _file );
    fclose( _file );
    free( _picture_buf );
    free( _outbuf );

    avcodec_close( &_avCodecContext );

}

void
OutFFMpegStream::initEncoder()
{
    /* find the mpeg1 video encoder */
    _avCodec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
    if (!_avCodec) {
        ABORT( "codec not found\n");
    }

    /* put sample parameters */
    _avCodecContext.bit_rate = 40000000;
    /* resolution must be a multiple of two */
    _avCodecContext.width = Global::render_width;  
    _avCodecContext.height = Global::render_height;
    /* frames per second */
    _avCodecContext.frame_rate = static_cast<int>( Global::fps * FRAME_RATE_BASE );
    /* emit one intra frame every ten frames */
    _avCodecContext.gop_size = 10; // FIXME  make configurable

    /* open it */
    if (avcodec_open( &_avCodecContext, _avCodec ) < 0) {
        ABORT( "could not open codec\n" );
    }
    
    /* the codec gives us the frame size, in samples */

    _file = fopen( getFileName().c_str() , "w" );
    if ( !_file ) {
        ABORT( "could not open "<<getFileName() );
    }
    
    int size = _avCodecContext.width * _avCodecContext.height;

    // FIXME   what size??  when free ??
    _picture_buf = (uint8_t*) malloc((size * 3) / 2); 
    _picture.data[0] = 0;
    _picture.data[1] = _picture.data[0] + size;
    _picture.data[2] = _picture.data[1] + size / 4;
    _picture.linesize[0] = _avCodecContext.width;
    _picture.linesize[1] = _avCodecContext.width / 2;
    _picture.linesize[2] = _avCodecContext.width / 2;

    return;
}


void
OutFFMpegStream::putFrame( Frame & f )
{
    f.getAVContext()->decodeVideo( f );

    _picture.data[0] = f.getVBuf()->getPixels();

// FIXME
//     int res = img_convert( &_picture, PIX_FMT_YUV420P,
//                            &p, PIX_FMT_RGB24, 
//                            Global::render_width, Global::render_height );
//     if ( res != 0 ) {
//         WARN( "Couldn't convert image" );
//         return;
//     }

    /* encode the image */
    int out_size = avcodec_encode_video( &_avCodecContext, _outbuf, OUTBUFSIZE, &_picture);
    fwrite(_outbuf, 1, out_size, _file);

    return;
}
