/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, Rolf.Dubitzky@gmx.net
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// STDC
#include <unistd.h>
#include <getopt.h>

// STDC++
#include <iostream>
using std::cout;
using std::endl;
using std::string;

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/pluginmgr.hh"

using namespace PIAVE;

void usage( int argc,  char * const * argv ) { 
    cout << "usage:  "<<argv[0]<<" [-v] [-d <dir>] [-d <dir>] [-l <lib>] [-l <lib>]"<<endl;
    cout << " -v <level>       level of verbosity (default: 0=quiet) " << endl;
    cout << " -d <dir>   search <dir> recursively for plugins" << endl;
    cout << " -l <lib>   load plugin library <lib>" << endl;
    exit( 1 ); 
}

int
main( int argc,  char * const * argv  )
{

    bool load_plugins_from_default_source = true;
    int n_plugins = 0;

    int c;
    opterr = 0;
    while ((c = getopt (argc, argv, "l:d:v:h")) != -1) {
        switch (c)
        {
          case 'v':
              PIAVE::Global::verbosity = atoi( optarg );
             break;
          case 'h':
              usage( argc, argv );
              break;
          case 'd':
              n_plugins += PluginMgr::openDLLsFromDir( optarg );
              load_plugins_from_default_source = false;
              break;
          case 'l':
              n_plugins += PluginMgr::openDLL( optarg ); 
              load_plugins_from_default_source = false;
              break;
          default:
              cout <<"unknown option: "<<(char)c<<endl;
              usage( argc, argv );
              break;
        }
    }

    if ( load_plugins_from_default_source ) {
        n_plugins += PluginMgr::openDLLsFromDefaultPath();
    }

    PluginMgr::listPlugins( cout );

    cout << "Found "<<n_plugins<<" plugins."<<endl;

    return 0;
}

